#!/usr/bin/env python

__author__ = "Andrea Fioraldi, Luigi Paolo Pileggi"
__copyright__ = "Copyright 2017, Carbonara Project"
__license__ = "BSD 2-clause"
__email__ = "andreafioraldi@gmail.com, willownoises@gmail.com"

import json
import base64
import hashlib
import binascii
import struct
import os
import r2handler
import idblib
import capstone
import status
import idb
import config
import matching
import subprocess

class ArchNotSupported(RuntimeError):
    pass

class BinaryInfo(object):
    def __init__(self, filename):
        '''
        BinaryInfo

        :param str filename: The filename of target binary
        '''

        print "[Retrieving basic info about binary]"
        #open the binary file and compute sha256 hash
        binfile = open(filename, "rb")
        hash_object = hashlib.md5(binfile.read())
        hex_dig = hash_object.hexdigest()
        
        self.data = {
            "program": {
                "md5": hex_dig,
                "filename": filename
            },
            "procs": [],
            "codebytes": {}
        }
        
        #open radare2 as subprocess
        self.r2 = r2handler.open(filename)
        
        #r2 cmd izzj : get strings contained in the binary in json
        print "1: getting strings list..."
        strings = self.r2.cmdj('izzj')["strings"]
        
        self.data["strings"] = []
        for strg in strings:
            s = {
                "val": strg["string"],
                "offset": strg["paddr"],
                "size": strg["size"],
                "type": strg["type"]
            }
            self.data["strings"].append(s)
        
        #r2 cmd Sj : get sections
        print "2: getting sections..."
        sections = self.r2.cmdj('iSj')
        
        self.data["sections"] = []
        for sec in sections:
            offset = sec["paddr"]
            size = sec["size"]
            
            #calculate md5 of the section
            binfile.seek(offset)
            hash_object = hashlib.md5(binfile.read(size))
            hex_dig = hash_object.hexdigest()
            
            s = {
                "name": sec["name"],
                "offset": offset,
                "size": size,
                "md5": hex_dig
            }
            self.data["sections"].append(s)
        
        binfile.close()
        
        print "3: calculating entropy..."
        self.data["entropy"] = self.r2.cmdj('p=ej') #TODO ??? must be rewritten!!! listen ML experts

    def __del__(self):
        if "r2" in self.__dict__:
            self.r2.quit()

    def addProc(self, name, asm, raw, insns_list, ops, offset, callconv, flow):
        '''
        generate a dictionary with the informations needed to describe a procedure and add it to the procedures list

        :param str name: The procedure name
        :param str asm: The disassembly with comments
        :param str raw: The bytes of the function
        :param str ops: List of first bytes of each instruction
        :param integer offset: The offset of function from the binary base address
        :param str callconv: The calling canvention of the procedure
        :param array<JumpInsn> flow: Array of jump (or call) object extracted from the code
        '''
        
        handler = matching.ProcedureHandler(raw, insns_list, offset, flow, self.arch)
        
        handler.handleFlow()
        
        handler.lift()
        
        proc = {
            "name": name,
            "raw": base64.b64encode(raw),
            "asm": asm,
            "offset": offset,
            "callconv": callconv,
            "apicalls": handler.api
        }

        proc["hash1"] = handler.api_hash.encode("hex")
        proc["hash2"] = handler.internals_hash.encode("hex")
        proc["hash3"] = handler.jumps_flow_hash.encode("hex")
        proc["hash4"] = handler.flow_hash.encode("hex")
        proc["hash5"] = handler.consts_hash.encode("hex")
        proc["hash6"] = handler.vex_code_hash.encode("hex")
        proc["hash7"] = hashlib.md5(str(ops)).digest().encode("hex")
        proc["full_hash"] = hashlib.md5(raw).hexdigest()
        self.data["procs"].append(proc)

    def addString(self, string):
        self.data["strings"].append(string)

    def toJson(self):
        #return str(self.data)
        return json.dumps(self.data, indent=2, ensure_ascii=True)

    def __str__(self):
        return self.toJson()

    def fromIdaDB(self, filename):
        '''
        Get information about binary stored in a IDA database

        :param str filename: The name of the IDA databse or its path
        '''
        
        if config.idacmd == None:
             raise NotImplementedError() #parse idb without calling IDA

        print "2: Waiting for IDA to parse database (this may take several minutes)..."
        file_ext = os.path.splitext(filename)[1]
        if file_ext == '.idb':
            process = subprocess.Popen(config.idacmd + ' -A -S"' + os.path.join(os.path.dirname(__file__), "idascript.py") + '" ' + filename)
        elif file_ext == '.i64':
            process = subprocess.Popen(config.ida64cmd + ' -A -S"' + os.path.join(os.path.dirname(__file__), "idascript.py") + '" ' + filename)
        else:
            raise RuntimeError('file not supported')
        process.wait()

        #getting data from idascript via json
        idadump = open('dump.json', 'r')
        data = json.load(idadump)

        print "2: getting file properties..."
        self.data['info'] = data['info']

        try:
            self.arch = matching.archFromIda(self.data["info"]["arch"], self.data["info"]["bits"], self.data["info"]["endian"])
        except:
            raise ArchNotSupported("arch %s not supported" % self.data["info"]["arch"])
        self.data["info"]["arch"] = self.arch.name
        
        print "3: getting imported libraries..."
        self.data['libs'] = data['libs']

        print "4: getting imported procedures names..."
        self.data['imports'] = data['imports']

        print "5: getting exported symbols..."
        self.data["exports"] = data['exports']

        print "6: getting assembly and other info about each procedure..."
        
        with status.Status(len(funcs_dict)) as bar:
            count = 0
            for func in data['procedures']:
                try:
                    fcn_name = func['name']
                    asm = func['asm']
                    fcn_bytes = func['raw_data']
                    insns_list = []
                    for ins in func['insns_list']:
                        insns_list.append(base64.b64decode(ins))
                    opcodes_list = func['ops']
                    fcn_offset = func['offset']
                    fcn_call_conv = func['callconv']
                    flow_insns = func['flow_insns']
                    self.addProc(fcn_name, asm, fcn_bytes, insns_list, opcodes_list.decode("hex"), fcn_offset, fcn_call_conv, flow_insns)
                except Exception as err:
                    print "error on function %s, skipped" % func["name"]
                count += 1
                bar.update(count)

        idadump.close()

        #clean up
        os.remove('dump.json')

    def _r2Task(self):
        '''
        Get info from the radare2 process
        '''

        #r2 cmd iIj : get info about binary in json
        print "2: getting file properties..."
        self.data["info"] = self.r2.cmdj('iIj')
        self.data["info"]["program_class"] = self.data["info"]["class"] #rename for the backend
        del self.data["info"]["class"]
        
        try:
            self.arch = matching.archFromR2(self.data["info"]["arch"], self.data["info"]["bits"], self.data["info"]["endian"])
        except:
            raise ArchNotSupported("arch %s not supported" % self.data["info"]["arch"])
        self.data["info"]["arch"] = self.arch.name
        
        #r2 cmd ilj : get imported libs in json
        print "3: getting imported libraries..."
        self.data["libs"] = self.r2.cmdj('ilj')
        
        #r2 cmd ilj : get imported functions in json
        print "4: getting imported procedures names..."
        imports = self.r2.cmdj('iij')
        
        self.data["imports"] = []
        for imp in imports:
            i = {
                "name": imp["name"],
                "addr": imp["plt"] #??? for PE binaries?
            }
            self.data["imports"].append(i)
        
        #r2 cmd ilj : get exported symbols in json
        print "5: getting exported symbols..."
        exports = self.r2.cmdj('iEj')
        
        self.data["exports"] = []
        for exp in exports:
            e = {
                "name": exp["name"],
                "offset": exp["paddr"],
                "size": exp["size"]
            }
            self.data["exports"].append(e)

        #r2 cmd aflj : get info about analyzed functions
        print "6: getting list of analyzed procedures..."
        funcs_dict = self.r2.cmdj('aflj')
        sym_imp_l = len("sym.imp") 
        
        print "7: getting assembly and other info about each procedure..."
        with status.Status(len(funcs_dict)) as bar:
            count = 0
            for func in funcs_dict:
                try:
                    #skip library symbols
                    if len(func["name"]) >= sym_imp_l and func["name"][:sym_imp_l] == "sym.imp":
                        continue
                    
                    fcn_offset = func["offset"]
                    fcn_size = func["size"]
                    fcn_name = func["name"]
                    fcn_call_conv = func["calltype"]
                    
                    #r2 cmd pdfj : get assembly from a function in json
                    fcn_instructions = self.r2.cmdj('pdfj @ ' + fcn_name)
                    
                    #r2 cmd p6e : get bytes of a function in base64
                    self.r2.cmd('s ' + str(fcn_offset))
                    fcn_bytes = base64.b64decode(self.r2.cmd('p6e ' + str(fcn_size)).rstrip())
                    
                    insns_list = []
                    asm = ""
                    opcodes_list = ""
                    
                    flow_insns = []
                    
                    for instr in fcn_instructions["ops"]:
                        if instr["type"] == "invalid":
                            break
                        
                        #get the first byte in hex
                        first_byte = instr["bytes"][:2]
                        opcodes_list += first_byte
                        
                        #insert ops in codebytes (field with the frequency of each opcode, useful for ML)
                        self.data["codebytes"][first_byte] = self.data["codebytes"].get(first_byte, 0) +1
                        
                        #insert comments in disassembly if presents
                        if "comment" in instr:
                            asm += instr["opcode"] + "  ; " + base64.b64decode(instr["comment"]) + "\n"
                        else:
                            asm += instr["opcode"] + "\n"
                            
                        #check if the instruction is of type 'call'
                        if instr["type"] == "call" and "jump" in instr:
                            target_name = instr["opcode"].split()[-1]
                            call_instr = None
                            if target_name[:sym_imp_l] == "sym.imp":
                                call_instr = matching.CallInsn(instr["offset"], instr["size"], instr["jump"], target_name[sym_imp_l +1:], True)
                            elif target_name[:len("sub.")] == "sub.":
                                call_instr = matching.CallInsn(instr["offset"], instr["size"], instr["jump"], target_name[len("sub."):], True)
                            else:
                                call_instr = matching.CallInsn(instr["offset"], instr["size"], instr["jump"], target_name)
                            flow_insns.append(call_instr)
                        #check if the instruction is of type 'jump'
                        elif (instr["type"] == "cjmp" or instr["type"] == "jmp") and "jump" in instr:
                            target = instr["jump"]
                            jumpout = target < fcn_offset or target >= fcn_offset + fcn_size
                            jump_instr = matching.JumpInsn(instr["offset"], instr["size"], target, jumpout)
                            flow_insns.append(jump_instr)
                        
                        insns_list.append(instr["bytes"].decode("hex"))
                    
                    self.addProc(fcn_name, asm, fcn_bytes, insns_list, opcodes_list.decode("hex"), fcn_offset, fcn_call_conv, flow_insns)
                except Exception as err:
                    print "error on function %s, skipped" % func["name"]
                count += 1
                bar.update(count)

    def fromR2Project(self, name):
        '''
        Get information about binary stored in a radare2 project

        :param str name: The name of the radare2 project or its path
        '''

        print "[Retrieving info from radare2 project]"
        
        #set project directory var in radare
        projdir = os.path.dirname(name)
        projname = os.path.basename(name)
        if projdir != "":
            projdir = os.path.expanduser(projdir)
            self.r2.cmd("e dir.projects=" + projdir)
        
        #r2 cmd Po : load project
        print "1: loading project..."
        out = self.r2.cmd("Po " + projname)
        if len(out) >= len("Cannot open project info") and out == "Cannot open project info":
            raise RuntimeError("cannot load radare2 project " + name)
        
        self._r2Task()

    def generateInfo(self):
        '''
        Grab basic informations about the binary from r2
        '''

        print "[Extracting info from binary]"
        
        #r2 cmd aa : analyze all
        print "1: analyzing all..."
        self.r2.cmd("aaa")
        
        self._r2Task()






#!/usr/bin/env python

import os
import sys
import time
import json
import config
if os.name == "posix":
    import fcntl
from subprocess import Popen, PIPE

class open:
    """Class representing a custom r2pipe connection with a running radare2 instance
    """
    def __init__(self, filename='', flags=[]):
        """Open a new r2 pipe
        Args:
            filename (str): path to filename
            flags (list of str): arguments, either in comapct form
                ("-wdn") or sepparated by commas ("-w","-d","-n")
        Returns:
            Returns an object with methods to interact with r2 via commands
        """

        cmd = [config.radare2, "-q0", filename]
        cmd = cmd[:1] + flags + cmd[1:]
        try:
            self.process = Popen(cmd, shell=False, stdin=PIPE, stdout=PIPE, stderr=PIPE)
        except:
            raise Exception("ERROR: Cannot find radare2 in PATH")
        self.process.stdout.read(1) # Reads initial \x00
        # make it non-blocking to speedup reading
        if os.name == "posix":
            self.nonblocking = True
            if self.nonblocking:
                fd = self.process.stdout.fileno()
                fl = fcntl.fcntl(fd, fcntl.F_GETFL)
                fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)
        else:
            self.nonblocking = False

    def _cmd(self, cmd):
        cmd = cmd.strip().replace("\n", ";")
        if sys.version_info >= (3,0):
            self.process.stdin.write(bytes(cmd+'\n'))
        else:
            self.process.stdin.write(cmd+'\n')
        self.process.stdin.flush()
        out = b''
        while True:
            if self.nonblocking:
                try:
                    foo = self.process.stdout.read(4096)
                except:
                    continue
            else:
                foo = self.process.stdout.read(1)
            if foo[-1] == b'\x00':
                out += foo[0:-1]
                break
            out += foo
        return out

    def quit(self):
        """Quit current r2pipe session and kill
        """
        self.cmd("q")
        if hasattr(self, 'process'):
            self.process.stdin.flush()
            self.process.terminate()
            self.process.wait()

    # r2 commands
    def cmd(self, cmd):
        """Run an r2 command return string with result
        Args:
            cmd (str): r2 command
        Returns:
            Returns an string with the results of the command
        """
        res = self._cmd(cmd)
        if res is not None:
            return res.strip()
        return None

    def cmdj(self, cmd):
        """Same as cmd() but evaluates JSONs and returns an object
        Args:
            cmd (str): r2 command
        Returns:
            Returns a Python object respresenting the parsed JSON
        """
        c = self.cmd(cmd)
        try:
            data = json.loads(self.cmd(cmd))
        except (ValueError, KeyError, TypeError) as e:
            sys.stderr.write ("r2pipe.cmdj.Error: %s\n"%(e))
            data = None
        return data




#!/usr/bin/env python

__author__ = "Andrea Fioraldi, Luigi Paolo Pileggi"
__copyright__ = "Copyright 2017, Carbonara Project"
__license__ = "BSD 2-clause"
__email__ = "andreafioraldi@gmail.com, willownoises@gmail.com"

import pyvex
import archinfo
import hashlib

r2_arch_map = {
    ("arm", 64): archinfo.ArchAArch64,
    ("arm", 32): archinfo.ArchARM,
    ("x86", 64): archinfo.ArchAMD64,
    ("avr", 16): archinfo.ArchAVR,
    ("mips", 32): archinfo.ArchMIPS32,
    ("mips", 64): archinfo.ArchMIPS64,
    ("ppc", 32): archinfo.ArchPPC32,
    ("ppc", 64): archinfo.ArchPPC64,
    ("x86", 32): archinfo.ArchX86
}

''' HOW TO GET ARCH IN IDAPYTHON?
import idaapi

info = idaapi.get_inf_structure()

if info.is_64bit():
    bits = 64
elif info.is_32bit():
    bits = 32
else:
    bits = 16

endian = "little"
if info.mf:
    endian = "big"
'''

ida_arch_map = {
    ("arm", 64): archinfo.ArchAArch64,
    ("armb", 64): archinfo.ArchAArch64,
    ("arm", 32): archinfo.ArchARM,
    ("armb", 32): archinfo.ArchARM,
    ("metapc", 64): archinfo.ArchAMD64,
    ("avr", 16): archinfo.ArchAVR, #??? check if it is true in ida
    ("mips", 32): archinfo.ArchMIPS32,
    ("mipsb", 32): archinfo.ArchMIPS32,
    ("mips64", 64): archinfo.ArchMIPS64,
    ("mips64b", 64): archinfo.ArchMIPS64,
    ("ppc", 32): archinfo.ArchPPC32,
    ("ppcb", 32): archinfo.ArchPPC32,
    ("ppc64", 64): archinfo.ArchPPC64,
    ("ppc64b", 64): archinfo.ArchPPC64,
    ("metapc", 32): archinfo.ArchX86
    #TODO add more ida processors in the map
}

def archFromR2(arch, bits, endian):
    a = r2_arch_map[(arch, bits)]
    if endian != "little":
        return a(archinfo.Endness.BE)
    return a(archinfo.Endness.LE)

def archFromIda(processor, bits, endian):
    a = ida_arch_map[(arch, bits)]
    if endian != "little":
        return a(archinfo.Endness.BE)
    return a(archinfo.Endness.LE)


class JumpInsn(object):
    def __init__(self, offset, size, addr, jumpout=False):
        self.offset = offset
        self.size = size
        self.addr = addr
        self.jumpout = jumpout
    
    def __str__(self):
        return "JumpInsn(off:%x, siz:%d, tgt:%x, out:%r)" % (self.offset, self.size, self.addr, self.jumpout)

class CallInsn(JumpInsn):
    def __init__(self, offset, size, addr, fcn_name, is_api=False):
        JumpInsn.__init__(self, offset, size, addr, True)
        self.fcn_name = fcn_name
        self.is_api = is_api

    def __str__(self):
        return "CallInsn(off:%x, siz:%d, tgt:%x, name:%s, api:%r)" % (self.offset, self.size, self.addr, self.fcn_name, self.is_api)

class ProcedureHandler(object):
    def __init__(self, code, insns_list, offset, bb_insns, arch):
        self.insns_list = insns_list
        self.offset = offset
        self.bb_insns = bb_insns
        self.arch = arch
        self.code = code
        self.size = len(code)


    def liftByBlocks(self):
        bb = [0]
        for instr in self.bb_insns:
            if instr.jumpout:
                #if jump out continue with the next instruction
                next = instr.offset + instr.size - self.offset
                if next < self.size and next not in bb:
                    bb.append(next)
            else:
                if instr.addr - self.offset not in bb:
                    bb.append(instr.addr - self.offset)
                next = instr.offset + instr.size - self.offset
                if next < self.size and next not in bb:
                    bb.append(next)

        bb.sort()
        bb.append(self.offset + self.size)

        consts = {}
        ips = []
        vex_code = ""
        
        pc_offset = self.arch.registers["pc"][0]
        regs = {}
        irsbs = []

        for bidx in xrange(len(bb) -1):
            irsb = pyvex.IRSB(self.code[bb[bidx]:bb[bidx+1]], self.offset + bb[bidx], self.arch, opt_level=0)
            irsbs.append(irsb)
        
            stmts = irsb.statements
            
            for i in xrange(len(stmts)):
                #TODO PutI GetI
                
                if isinstance(stmts[i], pyvex.stmt.Put):
                    if stmts[i].offset == pc_offset and len(stmts[i].constants) == 1: 
                        ips.append(stmts[i].constants[0].value)
                        stmts[i].reg_name = "<>"
                    else:
                        # constants abstraction
                        for c in stmts[i].constants:
                            consts[c.value] = consts.get(c.value, len(consts))
                            c.value = consts[c.value]
                        
                    # registers abstraction
                    regs[stmts[i].offset] = regs.get(stmts[i].offset, len(regs))
                    stmts[i].offset = regs[stmts[i].offset]
                elif isinstance(stmts[i], pyvex.stmt.Exit):
                    ips.append(stmts[i].dst.value)
                    # registers abstraction
                    regs[stmts[i].offsIP] = regs.get(stmts[i].offsIP, len(regs))
                    stmts[i].offsIP = regs[stmts[i].offsIP]
                else:
                    # constants abstraction
                    for c in stmts[i].constants:
                        consts[c.value] = consts.get(c.value, len(consts))
                        c.value = consts[c.value]
                for expr in stmts[i].expressions:
                    if isinstance(expr, pyvex.expr.Get):
                        # registers abstraction
                        regs[expr.offset] = regs.get(expr.offset, len(regs))
                        expr.offset = regs[expr.offset]

        #order addresses
        addrs = {}
        ips.sort()
        for i in xrange(len(ips)):
            addrs[ips[i]] = i
        
        for irsb in irsbs:
            stmts = irsb.statements
                     
            for i in xrange(len(stmts)):
                if isinstance(stmts[i], pyvex.stmt.IMark) or isinstance(stmts[i], pyvex.stmt.AbiHint):
                    continue
                
                if hasattr(stmts[i], "reg_name") and stmts[i].reg_name == "<>":
                    stmts[i].constants[0].value = addrs[stmts[i].constants[0].value]
                elif isinstance(stmts[i], pyvex.stmt.Exit):
                    stmts[i].dst.value = addrs[stmts[i].dst.value]
                
                vex_code += stmts[i].__str__() + "\n"
        
        self.consts = consts
        self.ips = ips
        self.vex_code = vex_code
        
        consts_list = sorted(consts, key=consts.get)
        
        self.consts_hash = hashlib.md5(str(consts_list)).digest()
        self.vex_code_hash = hashlib.md5(vex_code).digest()
        
        '''
        import json
        print "----- ProcedureHandler.lift() -----"
        print vex_code
        print
        print "REGS: " + json.dumps(regs, indent=2)
        print "CONSTS: " + json.dumps(consts, indent=2)
        print "IPS: " + json.dumps(ips, indent=2)
        print "-----------------------------------"
        '''


    def liftByInsns(self):
        consts = {}
        ips = []
        vex_code = ""
        
        pc_offset = self.arch.registers["pc"][0]
        regs = {}
        irsbs = []

        
        for instr in self.insns_list:
            irsb = pyvex.IRSB(instr, 0, self.arch, opt_level=0)
            irsbs.append(irsb)
        
            stmts = irsb.statements
            
            for i in xrange(len(stmts)):
                #TODO PutI GetI
                
                if isinstance(stmts[i], pyvex.stmt.Put):
                    if stmts[i].offset == pc_offset and len(stmts[i].constants) == 1: 
                        ips.append(stmts[i].constants[0].value)
                        stmts[i].reg_name = "<>"
                    else:
                        # constants abstraction
                        for c in stmts[i].constants:
                            consts[c.value] = consts.get(c.value, len(consts))
                            c.value = consts[c.value]
                        
                    # registers abstraction
                    regs[stmts[i].offset] = regs.get(stmts[i].offset, len(regs))
                    stmts[i].offset = regs[stmts[i].offset]
                elif isinstance(stmts[i], pyvex.stmt.Exit):
                    ips.append(stmts[i].dst.value)
                    # registers abstraction
                    regs[stmts[i].offsIP] = regs.get(stmts[i].offsIP, len(regs))
                    stmts[i].offsIP = regs[stmts[i].offsIP]
                else:
                    # constants abstraction
                    for c in stmts[i].constants:
                        consts[c.value] = consts.get(c.value, len(consts))
                        c.value = consts[c.value]
                for expr in stmts[i].expressions:
                    if isinstance(expr, pyvex.expr.Get):
                        # registers abstraction
                        regs[expr.offset] = regs.get(expr.offset, len(regs))
                        expr.offset = regs[expr.offset]

        #order addresses
        addrs = {}
        ips.sort()
        for i in xrange(len(ips)):
            addrs[ips[i]] = i
        
        for irsb in irsbs:
            stmts = irsb.statements
                     
            for i in xrange(len(stmts)):
                if isinstance(stmts[i], pyvex.stmt.IMark) or isinstance(stmts[i], pyvex.stmt.AbiHint):
                    continue
                
                if hasattr(stmts[i], "reg_name") and stmts[i].reg_name == "<>":
                    stmts[i].constants[0].value = addrs[stmts[i].constants[0].value]
                elif isinstance(stmts[i], pyvex.stmt.Exit):
                    stmts[i].dst.value = addrs[stmts[i].dst.value]
                
                vex_code += stmts[i].__str__() + "\n"
        
        self.consts = consts
        self.ips = ips
        self.vex_code = vex_code
        
        consts_list = sorted(consts, key=consts.get)
        
        self.consts_hash = hashlib.md5(str(consts_list)).digest()
        self.vex_code_hash = hashlib.md5(vex_code).digest()
        
        '''
        import json
        print "----- ProcedureHandler.lift() -----"
        print vex_code
        print
        print "REGS: " + json.dumps(regs, indent=2)
        print "CONSTS: " + json.dumps(consts, indent=2)
        print "IPS: " + json.dumps(ips, indent=2)
        print "-----------------------------------"
        '''

    def lift(self):
        try:
            self.liftByBlocks()
        except pyvex.errors.PyVEXError:
            self.liftByInsns()       


    def handleFlow(self):
        
        #TODO replace sorting loops with sorted function
        api = []
        internals = []
        jumps = []
        api_str = ""

        for instr in self.bb_insns:
            if isinstance(instr, CallInsn):
                if instr.is_api:
                    api_str += str(instr.fcn_name) + ","
                    api.append(instr.fcn_name)
                else:
                    internals.append(instr.addr)
            else:
                if not instr.jumpout:
                    jumps.append(instr.addr)
        
        self.api_hash = hashlib.md5(api_str).digest()
        
        jumps.sort()
        jumps_dict = {}
        for i in xrange(len(jumps)):
            jumps_dict[jumps[i]] = i
        
        sorted = internals[:]
        sorted.sort()
        calleds_dict = {}
        for i in xrange(len(sorted)):
            calleds_dict[sorted[i]] = i
        for i in xrange(len(internals)):
            internals[i] = calleds_dict[internals[i]]
        
        self.jumps_flow = []
        self.flow = []
        jumps_flow_str = ""
        flow_str = ""
        
        for instr in self.bb_insns:
            if isinstance(instr, CallInsn):
                if instr.is_api:
                    self.flow.append(instr.fcn_name)
                    flow_str += "@" + instr.fcn_name + ","
            else:
                if not instr.jumpout:
                    self.flow.append(jumps_dict[instr.addr])
                    self.jumps_flow.append(jumps_dict[instr.addr])
                    jumps_flow_str += str(jumps_dict[instr.addr]) + ","
                    flow_str += str(jumps_dict[instr.addr]) + ","
        
        internals_str = ""
        for fn in internals:
            internals_str += str(fn) + ","
        
        self.internals_hash = hashlib.md5(internals_str).digest()
        self.jumps_flow_hash = hashlib.md5(jumps_flow_str).digest()
        self.flow_hash = hashlib.md5(flow_str).digest()
        self.api = api
        
        '''
        import json
        print "----- ProcedureHandler.handleFlow() -----"
        print "API: " + json.dumps(self.api, indent=2)
        print "INTERNALS: " + json.dumps(internals, indent=2)
        print "JUMPS FLOW: " + json.dumps(self.jumps_flow, indent=2)
        print "FLOW: " + json.dumps(self.flow, indent=2)
        print "-----------------------------------"
        '''




        




#!/usr/bin/env python

__author__ = "Andrea Fioraldi, Luigi Paolo Pileggi"
__copyright__ = "Copyright 2017, Carbonara Project"
__license__ = "BSD 2-clause"
__email__ = "andreafioraldi@gmail.com, willownoises@gmail.com"

from setuptools import setup
import os

with open('README.md') as readme_file:
    readme = readme_file.read()

capstone_pkg = "capstone"
if os.name == 'nt':
    capstone_pkg = "capstone-windows"

setup(
    name='guanciale',
    version="1.0alpha",
    license=__license__,
    description='Grab information needed by Carbonara',
    long_description=readme,
    author=__author__,
    author_email=__email__,
    url='https://github.com/Carbonara-Project/Guanciale',
    download_url = 'https://github.com/Carbonara-Project/Guanciale/archive/1.0alpha.tar.gz',
    package_dir={'guanciale': 'guanciale'},
    packages=['guanciale'],
    install_requires=[
        capstone_pkg,
        'requests',
        'python-idb',
        'archinfo',
        'pyvex'
    ]
)
